@@ -30,7 +30,6 @@
 #include "filter.h"
 #include "PTcommon.h"
 #include "ColourBrightness.h"
-
 #include "pttiff.h"
 #include "file.h"
 #include "PTcommon.h"
@@ -38,6 +37,7 @@
 
 #include <assert.h>
 #include <float.h>
+#include <math.h>
 
 //#include <stdio.h>
 //#include <stdlib.h>
@@ -55,6 +55,10 @@
 #endif
 
 
+// Uncomment following line to enable testing of inverses in getROI
+#define PANO_TEST_INVERSE
+
+
 int panoFlattenTIFF(fullPath * fullPathImages, int counterImageFiles,
                     fullPath * outputFileName, int removeOriginals);
 
@@ -220,7 +224,7 @@ int panoPSDCreate(fullPath * fullPathImages, int numberImages,
 #endif
 
 
-static void ARGtoRGBAImage(Image * im)
+void ARGtoRGBAImage(Image * im)
 {
     int right;
     int left;
@@ -465,11 +469,14 @@ void getROI(TrformStr * TrPtr, aPrefs * aP, PTRect * ROIRect)
 {
     struct MakeParams mpinv;
     fDesc invstack[15], finvD;
+    struct MakeParams mp;
+    fDesc stack[15], fD;
     int color = 0;
 
     int x, y, x_jump;
     double x_d, y_d;            // Cartesian Coordinates of point in source (i.e. input) image
     double Dx, Dy;              // Coordinates of corresponding point in destination (i.e. output) image
+    double Dx2, Dy2;              // Coordinates of corresponding point in destination (i.e. output) image
 
     double w2 = (double) TrPtr->dest->width / 2.0 - 0.5;        //half destination image width
     double h2 = (double) TrPtr->dest->height / 2.0 - 0.5;       //half destination image height
@@ -482,14 +489,20 @@ void getROI(TrformStr * TrPtr, aPrefs * aP, PTRect * ROIRect)
     ROIRect->top = TrPtr->dest->height - 1;
     ROIRect->bottom = 0;
 
-    //The "forward" transform (although not used here) allows us to map pixel
+    //The "forward" transform allows us to map pixel
     //coordinates in the output image to their location in the source image.
-    //SetMakeParams( stack, &mp, &(aP->im) , &(aP->pano), color );
-    //fD.func = execute_stack; fD.param = stack;
+    // We use it to test the inverse functions of libpano
+    printf("-------------------------------------------\n");
+    SetMakeParams( stack, &mp, &(aP->im) , &(aP->pano), color );
+    fD.func = execute_stack_new; 
+    fD.param = stack;
 
     //The "inverse" transform allows us to map pixel coordinates in each source image
     //to their location in the output image.
+    //    printf("INV-------------------------------------------\n");
     SetInvMakeParams(invstack, &mpinv, &(aP->im), &(aP->pano), color);
+
+    //    printf("-------------------------------------------\n");
     finvD.func = execute_stack_new;
     finvD.param = invstack;
 
@@ -501,45 +514,129 @@ void getROI(TrformStr * TrPtr, aPrefs * aP, PTRect * ROIRect)
     //needed here, but for now include some representative interior points as well.
     for (y = 0; y <= TrPtr->src->height; y += 1) {
         
-                x_jump = (y==0 || y==TrPtr->src->height || abs(y - TrPtr->src->height/2)<=5) ? 1 : TrPtr->src->width/2; 
-                
+        x_jump = (y==0 || y==TrPtr->src->height || abs(y - TrPtr->src->height/2)<=5) ? 1 : TrPtr->src->width/2; 
+
                 for (x = 0; x <= TrPtr->src->width; x += x_jump) {
                         //convert source coordinates to cartesian coordinates (i.e. origin at center of image)
                         x_d = (double) x - sw2 ;
                         y_d = (double) y - sh2 ;
-                        
+
                         //Map the source image cartesian coordinate to the destination image cartesian coordinate
                         finvD.func( x_d, y_d, &Dx, &Dy, finvD.param);
                         
+#ifdef PANO_TEST_INVERSE
+                        fD.func( Dx, Dy, &Dx2, &Dy2, fD.param);
+                        {
+                            int newX, newY;
+                            if (!isnan(Dx2) && !isnan(Dy2)) {
+
+                                newX = (int)(Dx2 + 0.5 + sw2);
+                                newY = (int)(Dy2 + 0.5 + sh2);
+
+                                if (newX != x || newY != y) {
+                                    printf("  X,Y: %7.1f,%7.1f (%5d,%5d) -> OUT: %9.1f, %9.1f inv -> %9.1f %9.1f (%5d, %5d) -- error %5d,%5d\n", 
+                                           x_d, y_d, 
+                                           x,y,
+                                           Dx, Dy, 
+                                           Dx2, Dy2, 
+                                           newX, newY,
+                                           newX - x,
+                                           newY - y
+                                           );
+                                }
+                                // If this assertion fails, there is an error. The question is, how big? See the values above.
+                                // it is possible that the error is so small that it does not matter.
+/*
+                                assert(fabs(newX-x) <= 20.0);
+                                assert(fabs(newY-y) <= 20.0);
+*/
+                            }
+
+                        }
+                          //printf("  IN1: %f,%f -> OUT: %f, %f inv -> %f %f    (%d, %d)\n", x_d, y_d, Dx, Dy, Dx2, Dy2, (int)(Dx +0.5), (int)(Dy+0.5));
+                          //printf("  IN3: %d,%d -> OUT: %f, %f   (%d, %d)\n", x, y, Dx, Dy, (int)(Dx +0.5), (int)(Dy+0.5));
+#endif
+                        
                         //Convert destination cartesian coordinate back to destination "screen" coordinates (i.e. origin at top left of image)
                         Dx += w2;
                         Dy += h2;
-                        
-                        //printf("  IN: %d,%d -> OUT: %f, %f   (%d, %d)\n", x, y, Dx, Dy, (int)Dx, (int)Dy);
-                        
+
                         //Expand ROI if necessary
                         //I've observed that in some cases, the mapping function returns
                         //a value of "-1.#IND00".  This is not a number, and probably indicates
                         //a divide by zero error somewhere in the mapping function.  This should
                         //be solved, but, for now, discard this value and keep going
                         if (!isnan(Dx)) {
-                                if ((int)Dx < ROIRect->left) ROIRect->left = (int)Dx;
-                                if ((int)Dx > ROIRect->right) ROIRect->right = (int)Dx;
+                            if ((int)Dx < ROIRect->left) ROIRect->left = (int)(Dx + 0.5);
+                            if ((int)Dx > ROIRect->right) ROIRect->right = (int)(Dx + 0.5);
                         }
                         if (!isnan(Dy)){                
-                                if ((int)Dy < ROIRect->top) ROIRect->top = (int)Dy;
-                                if ((int)Dy > ROIRect->bottom) ROIRect->bottom = (int)Dy;
+                            if ((int)Dy < ROIRect->top) ROIRect->top = (int)(Dy + 0.5);
+                            if ((int)Dy > ROIRect->bottom) ROIRect->bottom = (int)(Dy + 0.5);
                         }
                 }
         }
         
+    // printf("ROI1: %ld,%ld - %ld, %ld\n", ROIRect->left, ROIRect->top, ROIRect->right, ROIRect->bottom);
+
+        // If the destination is wrapped and the ROI is at least 95% of the width
+        // Then ensure it is of the full width so no missing data is removed from either side.
+        if ( (TrPtr->mode & _wrapX) &&
+             ( (ROIRect->right-ROIRect->left) > (TrPtr->dest->width * 0.95) ) )
+        {
+          ROIRect->left  = 0;
+          ROIRect->right = TrPtr->dest->width-1;
+        }
+
+        // Test for a zenith shot
+        if(ROIRect->top != 0)
+        {
+          Dx = 0;
+          Dy = - h2;
+          fD.func( Dx, Dy, &Dx2, &Dy2, fD.param);
+          {
+            if (!isnan(Dx2) && !isnan(Dy2)) {
+
+              x_d = (int)(Dx2 + 0.5 + sw2);
+              y_d = (int)(Dy2 + 0.5 + sh2);
+              // if x_d, y_d is inside the image then this is a zenith shot and should extent to the top
+              if (y_d >= 0 && y_d <= TrPtr->src->height &&
+                  x_d >= 0 && x_d <= TrPtr->src->width )
+              {
+                ROIRect->top = 0;
+              }
+            }
+          }
+        }
+
+        // Test for a nadir shot
+        if(ROIRect->bottom != TrPtr->dest->height-1)
+        {
+          Dx = 0;
+          Dy = h2;
+          fD.func( Dx, Dy, &Dx2, &Dy2, fD.param);
+          {
+            if (!isnan(Dx2) && !isnan(Dy2)) {
+
+              x_d = (int)(Dx2 + 0.5 + sw2);
+              y_d = (int)(Dy2 + 0.5 + sh2);
+              // if x_d, y_d is inside the image then this is a zenith shot and should extent to the top
+              if (y_d >= 0 && y_d <= TrPtr->src->height &&
+                  x_d >= 0 && x_d <= TrPtr->src->width )
+              {
+                ROIRect->bottom = TrPtr->dest->height-1;
+              }
+            }
+          }
+        }
+
         //Reduce ROI if it extends beyond boundaries of final panorama region
         if (ROIRect->left    < 0) ROIRect->left =0;
         if (ROIRect->top     < 0) ROIRect->top  =0;
         if (ROIRect->right   > (TrPtr->dest->width-1))  ROIRect->right    = TrPtr->dest->width-1;  
         if (ROIRect->bottom  > (TrPtr->dest->height-1)) ROIRect->bottom   = TrPtr->dest->height-1;
         
-        //printf("ROI: %d,%d - %d, %d\n", ROIRect->left, ROIRect->top, ROIRect->right, ROIRect->bottom);
+        //        printf("ROI2: %ld,%ld - %ld, %ld\n", ROIRect->left, ROIRect->top, ROIRect->right, ROIRect->bottom);
 }
 
 
@@ -621,7 +718,7 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
 
     int ebx;
 
-    int croppedTIFFIntermediate = 1;
+    int croppedTIFFIntermediate = 0;
     int croppedWidth = 0, croppedHeight = 0;
     PTRect ROIRect;
     unsigned int outputScanlineNumber = 0;
@@ -649,6 +746,7 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
 
     if ((regScript = LoadScript(scriptFileName)) == 0) {
         PrintError("Could not load ScriptFile");
+        fclose(regFile);
         goto mainError;
     }
 
@@ -660,6 +758,7 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
     // Make sure script was written completely
     if (regWritten != strlen(regScript)) {
         PrintError("Could not write temporary script");
+        fclose(regFile);
         goto mainError;
     }
     
@@ -673,7 +772,7 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
     memset(&transform, 0, sizeof(TrformStr));
     transform.src = &image1;    // Input image
     transform.dest = &resultPanorama;   // Output image
-    transform.mode = 8;         // How to run transformation
+    transform.mode = _honor_valid;      // How to run transformation
     transform.success = 1;      // 1 success 0 failure
 
     //Allocate space to hold fully qualified names of input images
@@ -731,13 +830,6 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
         //to ever disable this feature, other than for testing/debugging purposes.
 
 
-	// By default we do cropped TIFF,
-	// but at this point we cannot properly calculate the ROI for fisheyes, so just
-	// process them uncropped.
-
-	if (prefs->im.format == _fisheye_circ) {
-	    croppedTIFFIntermediate = 0;
-	} 
 
         colourCorrection = prefs->sBuf.colcorrect;
         // This is a strange value:
@@ -777,15 +869,41 @@ int panoCreatePanorama(fullPath ptrImageFileNames[], int counterImageFiles,
         --tempString;           /* nextWord does ++ before testing anything, this guarantess proper execution */
         nextWord(output_file_format, &tempString);
         
-	if (loopCounter == 0 && strcmp(output_file_format, "TIFF_m") != 0) {
+	if (strcmp(output_file_format, "TIFF_m") == 0 ) {
+            // CHeck if we are suppose to do cropped or uncropped
+            croppedTIFFIntermediate = 1;
+            if(strcmp(tempString, "") != 0) {
+              nextWord(output_file_format, &tempString);
+              if (strcmp(output_file_format, "") == 0 || 
+                  strcmp(output_file_format, "r:CROP") == 0
+                  ) {
+                  // DO uncropped by default unless we are fisheye
+                  if (prefs->im.format == _fisheye_circ) {
+                      PrintError("Cropped output is unsupported for circular fisheye lenses. Ignored");
+                  } 
+              } else if (strcmp(output_file_format, "r:UNCROP") == 0) {
+                  croppedTIFFIntermediate = 0;
+              } else {
+                  PrintError("Unsupported option in TIFF_m output (%s). Ignored", output_file_format);
+              }
+            }
+            // DO uncropped by default unless we are fisheye
+            if (prefs->im.format == _fisheye_circ) {
+                croppedTIFFIntermediate = 0;
+            } 
+        } else {
 	    PrintError("No support for this ouput image format (%s). Output will be TIFF_m", output_file_format);
 	}
+        // enable this to avoid cropped tiffs. usually for testing
+        //croppedTIFFIntermediate = 0;
 
 
 
         transform.interpolator  = prefs->interpolator;
         transform.gamma         = prefs->gamma;
         transform.fastStep      = prefs->fastStep;
+        if( prefs->pano.hfov == 360.0 )
+          transform.mode       |= _wrapX;
         
         if (ptQuietFlag == 0) {
             sprintf(tmpStr, "Converting Image %d / %d", (loopCounter + 1),
@@ -1687,8 +1805,8 @@ int panoCroppingMain(int argc,char *argv[], int operation, char *version, char *
     pano_cropping_parms croppingParms;
     char outputPrefix[MAX_PATH_LENGTH];
     int ptDeleteSources = 0;
-    fullPath *ptrInputFiles;
-    fullPath *ptrOutputFiles;
+    fullPath *ptrInputFiles = NULL;
+    fullPath *ptrOutputFiles = NULL;
     int base;
     int i;
   
@@ -1696,7 +1814,7 @@ int panoCroppingMain(int argc,char *argv[], int operation, char *version, char *
     strcpy(outputPrefix, defaultPrefix);
     bzero(&croppingParms, sizeof(croppingParms));
     
-    printf(version);
+    printf("%s", version);
     
     //Need enough space for a message to be returned if something goes wrong
   
@@ -1725,7 +1843,7 @@ int panoCroppingMain(int argc,char *argv[], int operation, char *version, char *
             ptQuietFlag = 1;
             break;
         case 'h':
-            printf(usage);
+            printf("%s",usage);
             exit(0);
         default:
             break;
@@ -1735,13 +1853,15 @@ int panoCroppingMain(int argc,char *argv[], int operation, char *version, char *
 
     if (filesCount < 1) {
         PrintError("No files specified in the command line");
-        printf(usage);
+        printf("%s",usage);
         exit(0);
     }
     // Allocate memory for filenames
     if ((ptrInputFiles = calloc(filesCount, sizeof(fullPath))) == NULL || 
         (ptrOutputFiles = calloc(filesCount, sizeof(fullPath))) == NULL)        {
         PrintError("Not enough memory");
+	      free(ptrInputFiles);
+	      free(ptrOutputFiles);
         return -1;
     }
 
@@ -1806,3 +1926,37 @@ int panoCroppingMain(int argc,char *argv[], int operation, char *version, char *
 
     return 0;
 }
+
+
+
+void panoPrintImage(char *msg, Image *im)
+{
+    printf("-------------%s\n", msg);
+    if (im != NULL) {
+        printf(">>>Image format %d\n", im->format);
+        printf(">>>Roll %f\n", im->roll);
+        printf(">>>Pitch %f\n", im->pitch);
+        printf(">>>Yaw %f\n", im->yaw);
+        
+        printf(">>>im->cP.shear %d\n", im->cP.shear);
+        printf(">>>im->cP.tilt %d\n", im->cP.tilt);
+        printf(">>>im->cP.tilt_x %f\n", im->cP.tilt_x);
+        printf(">>>im->cP.tilt_y %f\n", im->cP.tilt_y);
+        printf(">>>im->cP.tilt_z %f\n", im->cP.tilt_z);
+        printf(">>>im->cP.tilt_scale %f\n", im->cP.tilt_scale);
+        
+        printf(">>>im->cP.translation %d\n", im->cP.trans);
+        printf(">>>im->cP.trans_x %f\n", im->cP.trans_x);
+        printf(">>>im->cP.trans_y %f\n", im->cP.trans_y);
+        printf(">>>im->cP.trans_z %f\n", im->cP.trans_z);
+        
+        printf(">>>im->cP.test %d\n", im->cP.test);
+        printf(">>>im->cP.test parm1 %f\n", im->cP.test_p0);
+        printf(">>>im->cP.test parm2 %f\n", im->cP.test_p1);
+        printf(">>>im->cP.test parm3 %f\n", im->cP.test_p2);
+        printf(">>>im->cP.test parm4 %f\n", im->cP.test_p3);
+    }
+    printf("\n\n");
+
+}
+
