@@ -322,7 +322,7 @@ void queryFeatures(int index,char** name,Tp12FeatureType* type)
 //////////////////////////////////////////////////////////////////////
 
 
-char *panoFormatNames[PANO_FORMAT_COUNT] = {
+char *panoFormatNames[] = {
     "Rectilinear",
     "Cylindrical",
     "Equirectangular",
@@ -340,11 +340,12 @@ char *panoFormatNames[PANO_FORMAT_COUNT] = {
     "Orthographic",
     "Equisolid",
     "Equirectangular Panini",
-	"Biplane",
-	"Triplane",
+    "Biplane",
+    "Triplane",
+    "Panini General",
 };
 
-static int panoFormatID[PANO_FORMAT_COUNT] = {
+static int panoFormatID[] = {
     _rectilinear,
     _panorama,
     _equirectangular,
@@ -362,22 +363,24 @@ static int panoFormatID[PANO_FORMAT_COUNT] = {
     _orthographic,
     _equisolid,
     _equipanini,
-	_biplane,
-	_triplane,
+    _biplane,
+    _triplane,
+    _panini_general,
     };
 
 
 int panoProjectionFormatCount(void)
 {
-    // REturn the number of Projection formats available in the library
-    assert(sizeof(panoFormatNames) == PANO_FORMAT_COUNT * sizeof(char*));
+    // Return the number of Projection formats available in the library
+    assert(sizeof(panoFormatNames) == PANO_FORMAT_COUNT * sizeof(typeof (panoFormatNames[0])));
     return PANO_FORMAT_COUNT;
 }
 
 int panoProjectionFeaturesQuery(int projection, pano_projection_features *features)
 {
-    // Return information regarding the characteristics of each of the projections 
-    // in the library.
+    // Return static information on the characteristics of each of the projections 
+    // in the library.  This now includes min, max and defualt parameter values.
+	// All data are zero by default.
 
     int i;
 
@@ -416,9 +419,31 @@ int panoProjectionFeaturesQuery(int projection, pano_projection_features *featur
     case PANO_FORMAT_EQUI_PANINI:
 	features->maxVFOV = 179;
 	features->maxHFOV = 359;
+        break;
+    case PANO_FORMAT_PANINI_GENERAL:
+	features->maxVFOV = 160;
+	features->maxHFOV = 320;
+	features->numberOfParameters = 3;
+	features->parm[0].name = "Cmpr";
+	features->parm[1].name = "Tops";
+	features->parm[2].name = "Bots";
+    features->parm[0].minValue = 0;
+    features->parm[0].maxValue = 150;
+	features->parm[0].defValue = 100;
+    features->parm[1].minValue = -100;
+    features->parm[1].maxValue = 100;
+    features->parm[2].minValue = -100;
+    features->parm[2].maxValue = 100;
+
         break;
     case PANO_FORMAT_FISHEYE_FF:
+	features->maxVFOV = 360;
+	features->maxHFOV = 360;
+	break;
     case PANO_FORMAT_ORTHOGRAPHIC:
+	features->maxVFOV = 180;
+	features->maxHFOV = 180;
+	break;
     case PANO_FORMAT_EQUISOLID:
 	features->maxVFOV = 360;
 	features->maxHFOV = 360;
@@ -446,6 +471,9 @@ int panoProjectionFeaturesQuery(int projection, pano_projection_features *featur
 	    features->parm[i].minValue = -90;
 	    features->parm[i].maxValue = +90;
 	}
+	features->parm[0].defValue = 0;
+	features->parm[1].defValue = 60;
+
 	break;
 	case PANO_FORMAT_BIPLANE:
         features->maxVFOV = 179;
@@ -454,6 +482,7 @@ int panoProjectionFeaturesQuery(int projection, pano_projection_features *featur
 		features->parm[0].name = "alpha";
 		features->parm[0].minValue=1;
 		features->parm[0].maxValue=179;
+		features->parm[0].defValue=45;
 		break;
 	case PANO_FORMAT_TRIPLANE:
 	    features->maxVFOV = 179;
@@ -462,6 +491,7 @@ int panoProjectionFeaturesQuery(int projection, pano_projection_features *featur
 		features->parm[0].name = "alpha";
 		features->parm[0].minValue=1;
 		features->parm[0].maxValue=120;
+		features->parm[0].defValue=60;
 		break;
     default:
 	assert(0); // A projection is missing!
@@ -470,5 +500,37 @@ int panoProjectionFeaturesQuery(int projection, pano_projection_features *featur
     return 1;
 }
 
+int queryFOVLimits( int projection,		/* projection index */
+				    double * params, /* length depends on projection */
+					double * lims	/* [0] = maxhfov, [1] = maxvfov */
+				  )
+{
+	pano_projection_features pf;
+	int ok = panoProjectionFeaturesQuery(projection, &pf);
+	lims[0] = lims[1] = 0;
+	if( !ok ) return 0;
+  // default fixed values
+	lims[0] = pf.maxHFOV;
+	lims[1] = pf.maxVFOV;
+  // compute dynamic values
+    switch (projection) {
+    default:
+	break;
+    case PANO_FORMAT_PANINI_GENERAL:
+		maxFOVs_panini_general	(params, lims );
+    break;
+    case PANO_FORMAT_BIPLANE:
+        lims[0] = params[0] + 179;
+        break;
+    case PANO_FORMAT_TRIPLANE:
+        lims[0] = 2 * params[0] + 179;
+        break;
+    case PANO_FORMAT_ALBERS_EQUAL_AREA_CONIC:
+	break;
+    }
+
+	return 1;
+}
+
 
 
