@@ -35,7 +35,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <dirent.h>
 
 #ifndef _MSC_VER
 #include <unistd.h>
@@ -389,7 +388,7 @@ int main(int argc,char *argv[])
 char* Filename(fullPath* path)
 {
     char *temp;
-    if ((temp = strrchr(path->name, '/')) != NULL) {
+    if ((temp = strrchr(path->name, PATH_SEP)) != NULL) {
 	temp++;
     } else {
 	temp = path->name;
